﻿''' <summary>
''' ペイロードのデータ形式の表現
''' 
''' 0x80形式のペイロード
''' 
''' データフォーマット
''' 
''' 3: 1バイト : 書式バージョン (0x01 固定, 将来のための拡張)
''' 4: 1バイト : IO状態 
'''    b7..b3b2b1b0とした場合 b0/b1/b2/b3 が DO1/DO2/DO3/DO4)の設定値と
'''    なり、0がHi、1がLoとなります。設定を有効化するために、続く IO状
'''    態マスクのビットがを1に設定します。
''' 5: 1バイト : IO状態設定マスク
'''    b7..b3b2b1b0とした場合 b0/b1/b2/b3 が DO1/DO2/DO3/DO4)の設定値と
'''    なり、0で対応するDOを設定しない、1で設定します。
''' 6: 2バイト : PWM1の設定値
''' 0(0%)～1024(100%)または0xFFFF(設定しない)を与えます。
''' 7: 2バイト : PWM2の設定値
''' 8: 2バイト : PWM3の設定値
''' 9: 2バイト : PWM4の設定値
''' 
''' 
''' (1,2,10は clsDataPresentationAscii で表現する)
''' 1: 1バイト : 宛先アドレス（論理デバイスID） (0x00: 親機, 0x01 ～ 0x64: 子機ID指定, 0x78: 全子機)
'''    親機から子機、または子機から親機への伝送に限ります。
''' 2: 1バイト : コマンド番号 (0x80 固定)
''' 10:1バイト : チェックサム
'''
''' 例
''' :7880 010F0F0380030002800200 DF
'''       ^3^4^5^^^6^^^7^^^8^^^9
''' 
''' 論理デバイスIDが0x78の子機の全てのデジタル出力（DO）をLOに設定し、
''' PWMのDUTY比を 0x380, 0x300, 0x280, 0x200 に設定します。
''' </summary>
''' <remarks></remarks>
Public Class ClsDataPayloadStdApp_0x80
    Implements ITweDataPayload

    Public Const _BY_PKT_LEN As Byte = 11
    Public Const _BY_PKT_ID As Byte = &H80

    Dim _bStored As Boolean
    Dim _iBtnMask As UShort
    Dim _iBtnUseMask As UShort
    Dim _iPWM(3) As UShort

    Public Function toByteArray() As Byte() Implements ITweDataPayload.toByteArray
        Dim b(_BY_PKT_LEN - 1) As Byte

        b(0) = &H1
        b(1) = _iBtnMask And &HFF
        b(2) = _iBtnUseMask And &HFF

        Dim i As Integer, j As Integer
        j = 3
        For i = 0 To 3
            b(j) = (_iPWM(i) And &HFF00) >> 8
            b(j + 1) = _iPWM(i) And &HFF
            j = j + 2
        Next

        toByteArray = b
    End Function

    Public Function parseByteArray(ByRef b() As Byte) As Boolean Implements ITweDataPayload.parseByteArray
        Dim i As Integer, j As Integer

        parseByteArray = False

        clearData() ' データ初期化
        If b.Length < _BY_PKT_LEN Then ' バイト列長のチェック
            Exit Function
        End If
        If b(0) <> &H1 Then ' プロトコルバージョンのチェック
            Exit Function
        End If

        _iBtnMask = b(1)
        _iBtnUseMask = b(2)

        j = 3
        For i = 0 To 3
            Dim h As UShort
            Dim l As UShort
            h = b(j)
            l = b(j + 1)
            _iPWM(i) = (h << 8) + l
            j = j + 2
        Next

        parseByteArray = True

        _bStored = True
    End Function

    Public Function getCmdId() As Byte Implements ITweDataPayload.getCmdId
        getCmdId = _BY_PKT_ID
    End Function

    Public Sub New()
        clearData()
    End Sub

    Public Sub New(ByRef _b As Byte())
        parseByteArray(_b)

        If Not _bStored Then
            Throw New Exception
        End If
    End Sub

    Public Sub New(ByVal iBtnMask As UShort, ByVal iBtnUseMask As UShort, ByRef iPWM As UShort())
        clearData()

        _iBtnMask = iBtnMask
        _iBtnUseMask = iBtnUseMask
        _iPWM = iPWM

        _bStored = True
    End Sub

    Private Sub clearData()
        Dim i As Integer

        _bStored = False
        _iBtnMask = 0
        _iBtnUseMask = 0

        For i = 0 To 3
            _iPWM(i) = 0
        Next
    End Sub

    Public Property iBtnMask As Integer
        Get
            Return _iBtnMask
        End Get
        Set(ByVal value As Integer)
            _iBtnMask = value
        End Set
    End Property

    Public Property iBtnUsedMask As Integer
        Get
            Return _iBtnUseMask
        End Get
        Set(ByVal value As Integer)
            _iBtnUseMask = value
        End Set
    End Property

    Public Property ausPWM(ByVal i As Integer) As UShort
        Get
            Return _iPWM(i)
        End Get
        Set(ByVal value As UShort)
            _iPWM(i) = value
        End Set
    End Property
End Class
