﻿''' <summary>
'''
''' 0x81形式のペイロード（IOデータの受信形式）
''' 
''' :7881 15017581000038002899000C04230000FFFFFFFFFF D4
'''  ^1^2 ^3^4^5^^^^^^^6^7^^^8^9^^^a^b^c^de1e2e3e4ef ^g
'''       +0+1+2+3+4+5+6+7+8+9+A+B+C+D+E+F+0+1+2+3+4
''' 
'''  3: パケット識別子 (アプリケーションIDより生成される)
'''  4: プロトコルバージョン (0x01 固定)
'''  5: LQI値、電波強度に応じた値で 0xFF が最大、0x00 が最小
'''  6: 送信元の個体識別番号
'''  7: 宛先の論理デバイスID
'''  8: タイムスタンプ (秒64カウント)
'''  9: 中継フラグ(中継済みなら1)
'''  a: 電源電圧[mV]
'''  b: 未使用
'''  c: DI の状態ビット。DI1(0x1) DI2(0x2) DI3(0x4) DI4(0x8)。1がOn(Lowレベル)。
'''  d: DI の変更状態ビット。DI1(0x1) DI2(0x2) DI3(0x4) DI4(0x8)。1が変更対象。
'''  e1～e4: AD1～AD4の変換値。0～2000[mV]のAD値を16で割った値を格納。
'''  ef: AD1～AD4の補正値　（LSBから順に２ビットずつ補正値、LSB側が　AD1, MSB側が AD4)
''' 
''' </summary>
''' <remarks></remarks>
Public Class ClsDataPayloadStdApp_0x81
    Implements ITweDataPayload

    Public Const _BY_PKT_LEN As Byte = 21
    Public Const _BY_PKT_ID As Byte = &H81

    Dim _bStored As Boolean
    Dim _iBtnMask As UShort
    Dim _iBtnUseMask As UShort
    Dim _iPWM(3) As UShort
    Dim _iVolt As UShort

    Dim _byPktIdent As Byte
    Dim _byLQI As Byte
    Dim _iAddrSender As UInteger
    Dim _byAddrDst As Byte
    Dim _iTimeStamp As UShort
    Dim _byRelayFlg As Byte

    Dim _byCkAppId As Byte
    Dim _byTemp As Byte

    ''' <summary>
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function toByteArray() As Byte() Implements ITweDataPayload.toByteArray
        Dim b(21 - 1) As Byte ' 21 バイトのデータ

        ' 読み出しのペイロードだが文字列にして出し入れしたりするので、実装しておく
        b(0) = _byCkAppId
        b(1) = 1
        b(2) = _byLQI
        set_BE_DWORD(_iAddrSender, b, 3)
        b(7) = _byAddrDst
        set_BE_WORD(_iTimeStamp, b, 8)
        b(&HA) = _byRelayFlg
        set_BE_WORD(iVolt, b, &HB)
        b(&HD) = _byTemp
        b(&HE) = _iBtnMask
        b(&HF) = _iBtnUseMask

        Dim j As Integer = &H10
        Dim e As Byte = 0
        For i = 0 To 3
            If _iPWM(i) = &HFFFF Then
                e = e + (3 << (i * 2))
                b(j) = &HFF
            Else
                e = e + (CByte((_iPWM(i) >> 2) And 3) << (i * 2))
                b(j) = CByte((_iPWM(i) >> 4) And &HFF)
            End If
            j = j + 1
        Next
        b(&H14) = e

        toByteArray = b
    End Function

    Public Function parseByteArray(ByRef b() As Byte) As Boolean Implements ITweDataPayload.parseByteArray
        Dim i As Integer, j As Integer

        parseByteArray = False

        clearData() ' データ初期化
        If b.Length < _BY_PKT_LEN Then ' バイト列長のチェック
            Exit Function
        End If
        If b(1) <> &H1 Then ' プロトコルバージョンのチェック
            Exit Function
        End If

        _byCkAppId = b(0)
        _byLQI = b(2)

        _iAddrSender = get_BE_DWORD(b, 3)
        _byAddrDst = b(7)

        _iTimeStamp = get_BE_WORD(b, 8)

        _byRelayFlg = b(&HA)

        _iVolt = get_BE_WORD(b, &HB)
        _byTemp = b(&HD)

        _iBtnMask = b(&HE)
        _iBtnUseMask = b(&HF)

        j = &H10
        For i = 0 To 3
            ' 補正項の計算
            Dim e As UShort
            e = (b(&H14) >> (i * 2)) And &H3
            e = e << 2

            If b(j) = &HFF Then
                ' FF の場合は未使用状態なので FFFF
                _iPWM(i) = &HFFFF
            Else
                _iPWM(i) = (CUShort(b(j)) << 4) + e
            End If
            j = j + 1
        Next

        parseByteArray = True
        _bStored = True
    End Function

    Public Function getCmdId() As Byte Implements ITweDataPayload.getCmdId
        getCmdId = _BY_PKT_ID
    End Function

    Public Sub New()
        clearData()
    End Sub

    Public Sub New(ByRef _b As Byte())
        parseByteArray(_b)

        If Not _bStored Then
            Throw New Exception
        End If
    End Sub

    Private Sub clearData()
        Dim i As Integer

        _bStored = False

        _iBtnMask = 0
        _iBtnUseMask = 0
        _iVolt = 0

        For i = 0 To 3
            _iPWM(i) = 0
        Next

        _byPktIdent = 0
        _byLQI = 0

        _iAddrSender = 0
        _byAddrDst = 0
        _iTimeStamp = 0
        _byRelayFlg = 0
    End Sub

    Public Property byRelayFlg As Byte
        Get
            Return _byRelayFlg
        End Get
        Set(ByVal value As Byte)
            _byRelayFlg = value
        End Set
    End Property

    Public Property iTimeStamp As UShort
        Get
            Return _iTimeStamp
        End Get
        Set(ByVal value As UShort)
            _iTimeStamp = value
        End Set
    End Property

    Public Property byAddrDst As Byte
        Get
            Return _byAddrDst
        End Get
        Set(ByVal value As Byte)
            _byAddrDst = value
        End Set
    End Property

    Public Property iAddrSender As UInteger
        Get
            Return _iAddrSender
        End Get
        Set(ByVal value As UInteger)
            _iAddrSender = value
        End Set
    End Property

    Public Property byLQI As Byte
        Get
            Return _byLQI
        End Get
        Set(ByVal value As Byte)
            _byLQI = value
        End Set
    End Property

    Public Property byPktIdent As Byte
        Get
            Return _byPktIdent
        End Get
        Set(ByVal value As Byte)
            _byPktIdent = value
        End Set
    End Property

    Public Property iVolt As UShort
        Get
            Return _iVolt
        End Get
        Set(ByVal value As UShort)
            _iVolt = value
        End Set
    End Property

    Public Property iBtnMask As UShort
        Get
            Return _iBtnMask
        End Get
        Set(ByVal value As UShort)
            _iBtnMask = value
        End Set
    End Property

    Public Property iBtnUsedMask As UShort
        Get
            Return _iBtnUseMask
        End Get
        Set(ByVal value As UShort)
            _iBtnUseMask = value
        End Set
    End Property

    Public Property ausPWM(ByVal i As Integer) As UShort
        Get
            Return _iPWM(i)
        End Get
        Set(ByVal value As UShort)
            _iPWM(i) = value
        End Set
    End Property
End Class
