﻿''' バイト列の表現形式
''' 
''' ITweDataFormat で出力されるバイト列に対して、ヘッダフッタ等を付加したコマンド形態への変形を行う
''' 
''' 例えば以下のようなデータ Format, Payload で示されるされるデータがあった場合、
'''   78 01 00 11 22 33 44 55
'''   ^^^^1 ^^^^^^^^^^^^^^^^2
'''       Format            Payload
''' このバイト列に対しての表現形式を定義する。 
''' 
''' 例えば clsDataPresentationAscii 形式では、以下のような文字列に表現される。
'''   ":7801001122334455??[CR][LF]"
''' 
'''   ?? はチェックサム
'''   [CR][LF] は 0D 0A の改行コード
'''
Public Class ClsDataPresentationAscii
    Implements ITweDataPresentation

    Dim _dataFormat As ITweDataFormat
    Dim _abyInput(1024) As Byte
    Dim _bPrev As Byte
    Dim _iLen As Integer
    Dim _iLrc As Integer
    Dim _eState As ITweDataPresentation.E_PARSE_STATE
    Dim _bStored As Boolean

    Dim _timeGot As Date

    Public Function parseByte(ByVal b As Byte) As ITweDataPresentation.E_PARSE_STATE Implements ITweDataPresentation.parseByte
        If _eState = ITweDataPresentation.E_PARSE_STATE.ERR Or _eState = ITweDataPresentation.E_PARSE_STATE.COMPLETE Then

            _eState = ITweDataPresentation.E_PARSE_STATE.EMPTY
        End If

        Select Case _eState
            Case ITweDataPresentation.E_PARSE_STATE.EMPTY
                ' 先頭の : の判定
                If b = Asc(":") Then
                    _eState = ITweDataPresentation.E_PARSE_STATE.READPAYLOAD1
                    _iLen = 0
                    _iLrc = 0
                End If

            Case ITweDataPresentation.E_PARSE_STATE.READPAYLOAD1
                ' HEXの読み出しまたは終端の検知
                If b = &HD Then ' CR
                    _eState = ITweDataPresentation.E_PARSE_STATE.READFOTTER
                Else
                    _eState = ITweDataPresentation.E_PARSE_STATE.READPAYLOAD2
                End If

            Case ITweDataPresentation.E_PARSE_STATE.READPAYLOAD2
                ' HEXの読み出し(下位)
                Try
                    _abyInput(_iLen) = Convert.ToByte(Convert.ToChar(_bPrev) & Convert.ToChar(b), 16)
                    _iLrc = _iLrc + _abyInput(_iLen)
                    _iLen = _iLen + 1
                    _eState = ITweDataPresentation.E_PARSE_STATE.READPAYLOAD1
                Catch
                    _eState = ITweDataPresentation.E_PARSE_STATE.ERR
                End Try

        End Select

        If _eState = ITweDataPresentation.E_PARSE_STATE.READFOTTER Then
            If (_iLrc And &HFF) = 0 Then ' LRC は 0 になっているはず

                'コピー先の配列を作成する 
                Dim abyDat(_iLen - 1 - 1) As Byte

                '指定された範囲をコピーする 
                Array.Copy(_abyInput, 0, abyDat, 0, _iLen - 1)

                ' Format/Payload を解析
                _dataFormat = genDataFormat(abyDat)

                _timeGot = Now

                _eState = ITweDataPresentation.E_PARSE_STATE.COMPLETE
            Else
                _eState = ITweDataPresentation.E_PARSE_STATE.ERR
                Dim byLrcCorrect As Byte
                byLrcCorrect = (Not (_iLrc - _abyInput(_iLen - 1))) + 1
                MsgBox("LRC should be &H" & Hex(byLrcCorrect))
            End If
        End If

        ' 直前の文字
        _bPrev = b

        ' 戻り値
        parseByte = _eState
    End Function

    Public Function toByteArray() As Byte() Implements ITweDataPresentation.toByteArray
        toByteArray = Nothing
        Dim bout As Byte()


        If Not IsNothing(_dataFormat) Then
            Dim b As Byte()

            b = _dataFormat.toByteArray()

            If Not IsArray(b) Then
                Exit Function
            End If

            If b.Length > 0 Then
                Dim i As Integer
                Dim j As Integer
                Dim lrc As Byte
                Dim h As Byte, l As Byte

                ' 配列の延長
                ReDim bout(b.Length * 2 + 5 - 1)

                ' １文字目は :
                bout(0) = Asc(":")

                ' データ形式への変換
                lrc = 0
                j = 1
                For i = 0 To b.Length - 1

                    lrc = lrc + b(i)

                    h = (b(i) And &HF0) >> 4
                    l = (b(i) And &HF)

                    bout(j) = Asc(Hex(h))
                    bout(j + 1) = Asc(Hex(l))

                    j = j + 2
                Next

                ' LRC の出力
                lrc = (Not (lrc)) + 1

                h = (lrc And &HF0) >> 4
                l = (lrc And &HF)

                bout(j) = Asc(Hex(h))
                bout(j + 1) = Asc(Hex(l))
                j = j + 2

                ' CR LF の出力
                bout(j) = &HD
                bout(j + 1) = &HA

                toByteArray = bout
            End If
        End If
    End Function

    ''' <summary>
    ''' メンバー変数の初期化
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub clearData()
        _dataFormat = Nothing
        _iLen = 0
        _bStored = False
        _eState = ITweDataPresentation.E_PARSE_STATE.EMPTY
    End Sub

    ''' <summary>
    ''' 初期化
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        clearData()
    End Sub

    ''' <summary>
    ''' バイト列による初期化
    ''' </summary>
    ''' <param name="b"></param>
    ''' <remarks></remarks>
    Public Sub New(ByRef b() As Byte)
        clearData()

        Dim i As Integer
        For i = 0 To b.Length - 1
            Dim stat As ITweDataPresentation.E_PARSE_STATE

            stat = parseByte(b(i))

            If stat = ITweDataPresentation.E_PARSE_STATE.COMPLETE Then
                Exit For
            End If
        Next
    End Sub

    Public Sub New(ByRef fmt As ITweDataFormat)
        clearData()
        _dataFormat = fmt
        _bStored = True
        _eState = ITweDataPresentation.E_PARSE_STATE.COMPLETE
    End Sub


    Public ReadOnly Property objFormat As ITweDataFormat Implements ITweDataPresentation.objFormat
        Get
            Return _dataFormat
        End Get
    End Property

    Public Property timeGot As Date Implements ITweDataPresentation.timeGot
        Get
            Return _timeGot
        End Get
        Set(ByVal value As Date)
            _timeGot = value
        End Set
    End Property

    ''' <summary>
    ''' String型に変換するメソッド (System.Object.ToString() をオーバーライドする)
    ''' ※ listBox などに格納したときに利用される。末尾にタイムスタンプを付加する。
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overrides Function ToString() As String
        Return System.Text.Encoding.ASCII.GetString(toByteArray()) & " (" & _timeGot.ToString & ")"
    End Function
End Class
