﻿''' バイト列の表現形式
''' 
''' ITweDataFormat で出力されるバイト列に対して、ヘッダフッタ等を付加したコマンド形態への変形を行う
''' 
''' 例えば以下のようなデータ Format, Payload で示されるされるデータがあった場合、
'''   78 01 00 11 22 33 44 55
'''   ^^^^1 ^^^^^^^^^^^^^^^^2
'''       Format            Payload
''' このバイト列に対しての表現形式を定義する。 
''' 
''' 例えば clsDataPresentationAscii 形式では、以下のような文字列に表現される。
'''   ":7801001122334455??[CR][LF]"
''' 
'''   ?? はチェックサム
'''   [CR][LF] は 0D 0A の改行コード
'''
Public Interface ITweDataPresentation
    ''' <summary>
    ''' 表現へのバイト列への変換
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function toByteArray() As Byte()

    ''' <summary>
    ''' １バイトを入力し、データを解釈する。
    ''' 
    ''' 空の ITweDataPresentation オブジェクトに対して、１バイトずつ解釈を進め、
    ''' １系列として解釈を行うまで実行する。E_PARSE_STATE.COMPLETE になれば完了。
    ''' 
    ''' 完了後 ITweDataFormat オブジェクトが完成している。
    ''' </summary>
    ''' <param name="b"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Function parseByte(ByVal b As Byte) As E_PARSE_STATE

    ''' <summary>
    ''' ITweDataFormat 形式のオブジェクトを得る
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    ReadOnly Property objFormat() As ITweDataFormat

    ''' <summary>
    ''' データ解釈完了のタイムスタンプ
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Property timeGot As Date

    ''' <summary>
    ''' 状態マシンの状態
    ''' </summary>
    ''' <remarks></remarks>
    Enum E_PARSE_STATE
        EMPTY
        READHEADER
        READHEADER1
        READHEADER2
        READPAYLOAD
        READPAYLOAD1
        READPAYLOAD2
        READLEN
        READLEN1
        READLEN2
        READCHECKSUM
        READCHECKSUM1
        READCHECKSUM2
        READFOTTER
        READFOTTER1
        READFOTTER2
        COMPLETE
        ERR
    End Enum

End Interface
